@@ -7,16 +7,20 @@
 # :license:   BSD (3 Clause), see LICENSE for more details.
 
 from __future__ import absolute_import, print_function, unicode_literals
+
 import os
 import re
 import sys
 from collections import namedtuple
 
-SERIES = 'rhubarb'
+# Lazy loading
+from . import local  # noqa
 
-__version__ = '4.3.1'
+SERIES = 'cliffs'
+
+__version__ = '4.4.3'
 __author__ = 'Ask Solem'
-__contact__ = 'ask@celeryproject.org'
+__contact__ = 'auvipy@gmail.com'
 __homepage__ = 'http://celeryproject.org'
 __docformat__ = 'restructuredtext'
 __keywords__ = 'task job queue distributed messaging actor'
@@ -24,7 +28,7 @@ __keywords__ = 'task job queue distributed messaging actor'
 # -eof meta-
 
 __all__ = (
-    'Celery', 'bugreport', 'shared_task', 'task',
+    'Celery', 'bugreport', 'shared_task', 'task', 'Task',
     'current_app', 'current_task', 'maybe_signature',
     'chain', 'chord', 'chunks', 'group', 'signature',
     'xmap', 'xstarmap', 'uuid',
@@ -149,10 +153,6 @@ def maybe_patch_concurrency(argv=None, short_opts=None,
         concurrency.get_implementation(pool)
 
 
-# Lazy loading
-from . import local  # noqa
-
-
 # this just creates a new module, that imports stuff on first attribute
 # access.  This makes the library faster to use.
 old_module, new_module = local.recreate_module(  # pragma: no cover
