@@ -7,7 +7,9 @@ from contextlib import contextmanager
 import pytest
 from case import ANY, Mock, call, patch, skip
 from kombu.serialization import prepare_accept_content
+from kombu.utils.encoding import ensure_bytes
 
+import celery
 from celery import chord, group, signature, states, uuid
 from celery.app.task import Context, Task
 from celery.backends.base import (BaseBackend, DisabledBackend,
@@ -29,6 +31,17 @@ class wrapobject(object):
         self.args = args
 
 
+class paramexception(Exception):
+
+    def __init__(self, param):
+        self.param = param
+
+
+class objectexception(object):
+    class Nested(Exception):
+        pass
+
+
 if sys.version_info[0] == 3 or getattr(sys, 'pypy_version_info', None):
     Oldstyle = None
 else:
@@ -92,6 +105,71 @@ class test_Backend_interface:
         assert list(b4.accept)[0] == 'application/x-yaml'
         assert prepare_accept_content(['yaml']) == b4.accept
 
+    def test_get_result_meta(self):
+        b1 = BaseBackend(self.app)
+        meta = b1._get_result_meta(result={'fizz': 'buzz'},
+                                   state=states.SUCCESS, traceback=None,
+                                   request=None)
+        assert meta['status'] == states.SUCCESS
+        assert meta['result'] == {'fizz': 'buzz'}
+        assert meta['traceback'] is None
+
+        self.app.conf.result_extended = True
+        args = ['a', 'b']
+        kwargs = {'foo': 'bar'}
+        task_name = 'mytask'
+
+        b2 = BaseBackend(self.app)
+        request = Context(args=args, kwargs=kwargs,
+                          task=task_name,
+                          delivery_info={'routing_key': 'celery'})
+        meta = b2._get_result_meta(result={'fizz': 'buzz'},
+                                   state=states.SUCCESS, traceback=None,
+                                   request=request, encode=False)
+        assert meta['name'] == task_name
+        assert meta['args'] == args
+        assert meta['kwargs'] == kwargs
+        assert meta['queue'] == 'celery'
+
+    def test_get_result_meta_encoded(self):
+        self.app.conf.result_extended = True
+        b1 = BaseBackend(self.app)
+        args = ['a', 'b']
+        kwargs = {'foo': 'bar'}
+
+        request = Context(args=args, kwargs=kwargs)
+        meta = b1._get_result_meta(result={'fizz': 'buzz'},
+                                   state=states.SUCCESS, traceback=None,
+                                   request=request, encode=True)
+        assert meta['args'] == ensure_bytes(b1.encode(args))
+        assert meta['kwargs'] == ensure_bytes(b1.encode(kwargs))
+
+    def test_get_result_meta_with_none(self):
+        b1 = BaseBackend(self.app)
+        meta = b1._get_result_meta(result=None,
+                                   state=states.SUCCESS, traceback=None,
+                                   request=None)
+        assert meta['status'] == states.SUCCESS
+        assert meta['result'] is None
+        assert meta['traceback'] is None
+
+        self.app.conf.result_extended = True
+        args = ['a', 'b']
+        kwargs = {'foo': 'bar'}
+        task_name = 'mytask'
+
+        b2 = BaseBackend(self.app)
+        request = Context(args=args, kwargs=kwargs,
+                          task=task_name,
+                          delivery_info={'routing_key': 'celery'})
+        meta = b2._get_result_meta(result=None,
+                                   state=states.SUCCESS, traceback=None,
+                                   request=request, encode=False)
+        assert meta['name'] == task_name
+        assert meta['args'] == args
+        assert meta['kwargs'] == kwargs
+        assert meta['queue'] == 'celery'
+
 
 class test_BaseBackend_interface:
 
@@ -191,6 +269,17 @@ class test_prepare_exception:
         y = self.b.exception_to_python(x)
         assert isinstance(y, Exception)
 
+    @pytest.mark.skipif(sys.version_info < (3, 3), reason='no qualname support')
+    def test_json_exception_nested(self):
+        self.b.serializer = 'json'
+        x = self.b.prepare_exception(objectexception.Nested('msg'))
+        assert x == {
+            'exc_message': ('msg',),
+            'exc_type': 'objectexception.Nested',
+            'exc_module': objectexception.Nested.__module__}
+        y = self.b.exception_to_python(x)
+        assert isinstance(y, objectexception.Nested)
+
     def test_impossible(self):
         self.b.serializer = 'pickle'
         x = self.b.prepare_exception(Impossible())
@@ -223,7 +312,7 @@ class KVBackend(KeyValueStoreBackend):
 
     def __init__(self, app, *args, **kwargs):
         self.db = {}
-        super(KVBackend, self).__init__(app)
+        super(KVBackend, self).__init__(app, *args, **kwargs)
 
     def get(self, key):
         return self.db.get(key)
@@ -321,9 +410,19 @@ class test_BaseBackend_dict:
         self.b._cache['task-exists'] = {'result': 'task'}
 
     def test_fail_from_current_stack(self):
+        import inspect
         self.b.mark_as_failure = Mock()
-        try:
+        frame_list = []
+
+        if (2, 7, 0) <= sys.version_info < (3, 0, 0):
+            sys.exc_clear = Mock()
+
+        def raise_dummy():
+            frame_str_temp = str(inspect.currentframe().__repr__)
+            frame_list.append(frame_str_temp)
             raise KeyError('foo')
+        try:
+            raise_dummy()
         except KeyError as exc:
             self.b.fail_from_current_stack('task_id')
             self.b.mark_as_failure.assert_called()
@@ -332,6 +431,15 @@ class test_BaseBackend_dict:
             assert args[1] is exc
             assert args[2]
 
+            if sys.version_info >= (3, 5, 0):
+                tb_ = exc.__traceback__
+                while tb_ is not None:
+                    if str(tb_.tb_frame.__repr__) == frame_list[0]:
+                        assert len(tb_.tb_frame.f_locals) == 0
+                    tb_ = tb_.tb_next
+            elif (2, 7, 0) <= sys.version_info < (3, 0, 0):
+                sys.exc_clear.assert_called()
+
     def test_prepare_value_serializes_group_result(self):
         self.b.serializer = 'json'
         g = self.app.GroupResult('group_id', [self.app.AsyncResult('foo')])
@@ -360,10 +468,27 @@ class test_BaseBackend_dict:
         b.mark_as_done('id', 10, request=request)
         b.on_chord_part_return.assert_called_with(request, states.SUCCESS, 10)
 
+    def test_mark_as_failure__bound_errback_eager(self):
+        b = BaseBackend(app=self.app)
+        b._store_result = Mock()
+        request = Mock(name='request')
+        request.delivery_info = {
+            'is_eager': True
+        }
+        request.errbacks = [
+            self.bound_errback.subtask(args=[1], immutable=True)]
+        exc = KeyError()
+        group = self.patching('celery.backends.base.group')
+        b.mark_as_failure('id', exc, request=request)
+        group.assert_called_with(request.errbacks, app=self.app)
+        group.return_value.apply.assert_called_with(
+            (request.id, ), parent_id=request.id, root_id=request.root_id)
+
     def test_mark_as_failure__bound_errback(self):
         b = BaseBackend(app=self.app)
         b._store_result = Mock()
         request = Mock(name='request')
+        request.delivery_info = {}
         request.errbacks = [
             self.bound_errback.subtask(args=[1], immutable=True)]
         exc = KeyError()
@@ -447,6 +572,26 @@ class test_BaseBackend_dict:
         b = BaseBackend(app=self.app)
         assert b.exception_to_python(None) is None
 
+    def test_exception_to_python_when_attribute_exception(self):
+        b = BaseBackend(app=self.app)
+        test_exception = {'exc_type': 'AttributeDoesNotExist',
+                          'exc_module': 'celery',
+                          'exc_message': ['Raise Custom Message']}
+
+        result_exc = b.exception_to_python(test_exception)
+        assert str(result_exc) == 'Raise Custom Message'
+
+    def test_exception_to_python_when_type_error(self):
+        b = BaseBackend(app=self.app)
+        celery.TestParamException = paramexception
+        test_exception = {'exc_type': 'TestParamException',
+                          'exc_module': 'celery',
+                          'exc_message': []}
+
+        result_exc = b.exception_to_python(test_exception)
+        del celery.TestParamException
+        assert str(result_exc) == "<class 't.unit.backends.test_base.paramexception'>([])"
+
     def test_wait_for__on_interval(self):
         self.patching('time.sleep')
         b = BaseBackend(app=self.app)
@@ -486,7 +631,11 @@ class test_KeyValueStoreBackend:
         self.b.forget(tid)
         assert self.b.get_state(tid) == states.PENDING
 
-    def test_store_result_parent_id(self):
+    @pytest.mark.parametrize('serializer',
+                             ['json', 'pickle', 'yaml', 'msgpack'])
+    def test_store_result_parent_id(self, serializer):
+        self.app.conf.accept_content = ('json', serializer)
+        self.b = KVBackend(app=self.app, serializer=serializer)
         tid = uuid()
         pid = uuid()
         state = 'SUCCESS'
@@ -509,6 +658,20 @@ class test_KeyValueStoreBackend:
         stored_meta = self.b.decode(self.b.get(self.b.get_key_for_task(tid)))
         assert stored_meta['group_id'] == request.group
 
+    def test_store_result_race_second_write_should_ignore_if_previous_success(self):
+        tid = uuid()
+        state = 'SUCCESS'
+        result = 10
+        request = Context(group='gid', children=[])
+        self.b.store_result(
+            tid, state=state, result=result, request=request,
+        )
+        self.b.store_result(
+            tid, state=states.FAILURE, result=result, request=request,
+        )
+        stored_meta = self.b.decode(self.b.get(self.b.get_key_for_task(tid)))
+        assert stored_meta['status'] == states.SUCCESS
+
     def test_strip_prefix(self):
         x = self.b.get_key_for_task('x1b34')
         assert self.b._strip_prefix(x) == 'x1b34'
@@ -549,6 +712,24 @@ class test_KeyValueStoreBackend:
         with pytest.raises(self.b.TimeoutError):
             list(self.b.get_many(tasks, timeout=0.01, interval=0.01))
 
+    def test_get_many_passes_ready_states(self):
+        tasks_length = 10
+        ready_states = frozenset({states.SUCCESS})
+
+        self.b._cache.clear()
+        ids = {uuid(): i for i in range(tasks_length)}
+        for id, i in items(ids):
+            if i % 2 == 0:
+                self.b.mark_as_done(id, i)
+            else:
+                self.b.mark_as_failure(id, Exception())
+
+        it = self.b.get_many(list(ids), interval=0.01, max_iterations=1, READY_STATES=ready_states)
+        it_list = list(it)
+
+        assert all([got_state['status'] in ready_states for (got_id, got_state) in it_list])
+        assert len(it_list) == tasks_length / 2
+
     def test_chord_part_return_no_gid(self):
         self.b.implements_incr = True
         task = Mock()
