@@ -8,6 +8,19 @@ from celery._state import _task_stack
 from celery.backends.rpc import RPCBackend
 
 
+class test_RPCResultConsumer:
+    def get_backend(self):
+        return RPCBackend(app=self.app)
+
+    def get_consumer(self):
+        return self.get_backend().result_consumer
+
+    def test_drain_events_before_start(self):
+        consumer = self.get_consumer()
+        # drain_events shouldn't crash when called before start
+        consumer.drain_events(0.001)
+
+
 class test_RPCBackend:
 
     def setup(self):
