@@ -94,6 +94,19 @@ class TasksCase:
 
         self.retry_task_noargs = retry_task_noargs
 
+        @self.app.task(bind=True, max_retries=3, iterations=0, shared=False)
+        def retry_task_without_throw(self, **kwargs):
+            self.iterations += 1
+            try:
+                if self.request.retries >= 3:
+                    return 42
+                else:
+                    raise Exception("random code exception")
+            except Exception as exc:
+                return self.retry(exc=exc, throw=False)
+
+        self.retry_task_without_throw = retry_task_without_throw
+
         @self.app.task(bind=True, max_retries=3, iterations=0,
                        base=MockApplyTask, shared=False)
         def retry_task_mockapply(self, arg1, arg2, kwarg=1):
@@ -351,6 +364,15 @@ class test_task_retries(TasksCase):
         finally:
             self.retry_task_mockapply.pop_request()
 
+    def test_retry_without_throw_eager(self):
+        assert self.retry_task_without_throw.apply().get() == 42
+
+    def test_retry_eager_should_return_value(self):
+        self.retry_task.max_retries = 3
+        self.retry_task.iterations = 0
+        assert self.retry_task.apply([0xFF, 0xFFFF]).get() == 0xFF
+        assert self.retry_task.iterations == 4
+
     def test_retry_not_eager(self):
         self.retry_task_mockapply.push_request()
         try:
@@ -1223,6 +1245,15 @@ class test_apply_async(TasksCase):
 
     def test_always_eager_with_task_serializer_option(self):
         self.app.conf.task_always_eager = True
+        self.app.conf.task_serializer = 'pickle'
+
+        @self.app.task
+        def task(*args, **kwargs):
+            pass
+        task.apply_async((1, 2, 3, 4, {1}))
+
+    def test_always_eager_uses_task_serializer_setting(self):
+        self.app.conf.task_always_eager = True
 
         @self.app.task(serializer='pickle')
         def task(*args, **kwargs):
